
namespace TestWeb.Jobs;

public static class JobApi
{
    public static void MapEndpoints(IEndpointRouteBuilder endpoints)
    {
        var group = endpoints.MapGroup("/api/jobs").WithTags("Jobs").AllowAnonymous();
        group.MapPost("/", Create).WithSummary("Create a job");
        group.MapGet("/", GetList).WithSummary("Get job list");
        group.MapGet("/{id}", GetById).WithSummary("Get job by id");
        group.MapPut("/{id}", UpdateById).WithSummary("Update job by id");
        group.MapDelete("/{id}", DeleteById).WithSummary("Delete job by id");
    }

    public record CreateRequest(string ClientName);

    public class CreateRequestValidator : AbstractValidator<CreateRequest>
    {
        public CreateRequestValidator()
        {
            RuleFor(x => x.ClientName).NotEmpty().MaximumLength(250);
        }
    }

    public record CreateResponse(int Id);

    public static async Task<Ok<CreateResponse>> Create(
        CreateRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var row = new Job
        {
            ClientName = request.ClientName,
            Pickup = "",
        };

        await db.Jobs.AddAsync(row, ct);
        await db.SaveChangesAsync(ct);

        var response = new CreateResponse(row.Id);
        return TypedResults.Ok(response);
    }


    public static async Task<Ok<Job[]>> GetList(AppDbContext db, CancellationToken ct)
    {
        var results = await db.Jobs.ToArrayAsync(ct);
        return TypedResults.Ok(results);
    }


    public static async Task<Results<Ok<Job>, NotFound>> GetById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Jobs
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        return result is null
            ? TypedResults.NotFound()
            : TypedResults.Ok(result);
    }


    public record UpdateByIdRequest(string ClientName);

    public class UpdateByIdRequestValidator : AbstractValidator<UpdateByIdRequest>
    {
        public UpdateByIdRequestValidator()
        {
            RuleFor(x => x.ClientName).NotEmpty().MaximumLength(250);
        }
    }

    public static async Task<Results<Ok, NotFound>> UpdateById(
        int id,
        UpdateByIdRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Jobs
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        if (result == null)
        {
            return TypedResults.NotFound();
        }

        result.ClientName = request.ClientName;
        result.LastUpdatedAtUtc = DateTime.UtcNow;
        await db.SaveChangesAsync(ct);

        return TypedResults.Ok();
    }


    public static async Task<Results<Ok, NotFound>> DeleteById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var rowsDeleted = await db.Jobs
            .Where(x => x.Id == id)
            .ExecuteDeleteAsync(ct);

        return rowsDeleted == 1
            ? TypedResults.Ok()
            : TypedResults.NotFound();
    }
}
