
namespace TestWeb.Notes;

public static class NoteApi
{
    public static void MapEndpoints(IEndpointRouteBuilder endpoints)
    {
        var group = endpoints.MapGroup("/api/notes").WithTags("Notes").AllowAnonymous();
        group.MapPost("/", Create).WithSummary("Create a note");
        group.MapGet("/", GetList).WithSummary("Get note list");
        group.MapGet("/{id}", GetById).WithSummary("Get note by id");
        group.MapPut("/{id}", UpdateById).WithSummary("Update note by id");
        group.MapDelete("/{id}", DeleteById).WithSummary("Delete note by id");
    }

    public record CreateRequest(string Content);

    public class CreateRequestValidator : AbstractValidator<CreateRequest>
    {
        public CreateRequestValidator()
        {
            RuleFor(x => x.Content).NotEmpty().MaximumLength(250);
        }
    }

    public record CreateResponse(int Id);

    public static async Task<Ok<CreateResponse>> Create(
        CreateRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var row = new Note
        {
            Content = request.Content
        };

        await db.Notes.AddAsync(row, ct);
        await db.SaveChangesAsync(ct);

        var response = new CreateResponse(row.Id);
        return TypedResults.Ok(response);
    } 


    public static async Task<Ok<Note[]>> GetList(AppDbContext db, CancellationToken ct)
    {
        var results = await db.Notes.ToArrayAsync(ct);
        return TypedResults.Ok(results);
    }


    public static async Task<Results<Ok<Note>, NotFound>> GetById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Notes
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        return result is null
            ? TypedResults.NotFound()
            : TypedResults.Ok(result);
    }


    public record UpdateByIdRequest(string Content);

    public class UpdateByIdRequestValidator : AbstractValidator<UpdateByIdRequest>
    {
        public UpdateByIdRequestValidator()
        {
            RuleFor(x => x.Content).NotEmpty().MaximumLength(250);
        }
    }

    public static async Task<Results<Ok, NotFound>> UpdateById(
        int id,
        UpdateByIdRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Notes
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        if (result == null)
        {
            return TypedResults.NotFound();
        }
        
        result.Content = request.Content;
        result.LastUpdatedAtUtc = DateTime.UtcNow;
        await db.SaveChangesAsync(ct);

        return TypedResults.Ok();
    }


    public static async Task<Results<Ok, NotFound>> DeleteById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var rowsDeleted = await db.Notes
            .Where(x => x.Id == id)
            .ExecuteDeleteAsync(ct);

        return rowsDeleted == 1
            ? TypedResults.Ok()
            : TypedResults.NotFound();
    }
}
