namespace TestWeb.Vehicles;

public static class VehicleApi
{
    public static void MapEndpoints(IEndpointRouteBuilder endpoints)
    {
        var group = endpoints.MapGroup("/api/vehicles").WithTags("Vehicles").AllowAnonymous();
        group.MapPost("/", Create).WithSummary("Create a vehicle");
        group.MapGet("/", GetList).WithSummary("Get vehicle list");
        group.MapGet("/{id}", GetById).WithSummary("Get vehicle by id");
        group.MapPut("/{id}", UpdateById).WithSummary("Update vehicle by id");
        group.MapDelete("/{id}", DeleteById).WithSummary("Delete vehicle by id");
    }

    public record CreateRequest(string Make);

    public class CreateRequestValidator : AbstractValidator<CreateRequest>
    {
        public CreateRequestValidator()
        {
            RuleFor(x => x.Make).NotEmpty().MaximumLength(250);
        }
    }

    public record CreateResponse(int Id);

    public static async Task<Ok<CreateResponse>> Create(
        CreateRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var row = new Vehicle
        {
            Make = request.Make,
            Model = "",
            Year = DateTime.Now.Year
        };

        await db.Vehicles.AddAsync(row, ct);
        await db.SaveChangesAsync(ct);

        var response = new CreateResponse(row.Id);
        return TypedResults.Ok(response);
    }


    public static async Task<Ok<Vehicle[]>> GetList(AppDbContext db, CancellationToken ct)
    {
        var results = await db.Vehicles.ToArrayAsync(ct);
        return TypedResults.Ok(results);
    }


    public static async Task<Results<Ok<Vehicle>, NotFound>> GetById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Vehicles
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        return result is null
            ? TypedResults.NotFound()
            : TypedResults.Ok(result);
    }


    public record UpdateByIdRequest(string Make);

    public class UpdateByIdRequestValidator : AbstractValidator<UpdateByIdRequest>
    {
        public UpdateByIdRequestValidator()
        {
            RuleFor(x => x.Make).NotEmpty().MaximumLength(250);
        }
    }

    public static async Task<Results<Ok, NotFound>> UpdateById(
        int id,
        UpdateByIdRequest request,
        AppDbContext db,
        CancellationToken ct)
    {
        var result = await db.Vehicles
            .Where(x => x.Id == id)
            .SingleOrDefaultAsync(ct);

        if (result == null)
        {
            return TypedResults.NotFound();
        }

        result.Make = request.Make;
        result.LastUpdatedAtUtc = DateTime.UtcNow;
        await db.SaveChangesAsync(ct);

        return TypedResults.Ok();
    }


    public static async Task<Results<Ok, NotFound>> DeleteById(
        int id,
        AppDbContext db,
        CancellationToken ct)
    {
        var rowsDeleted = await db.Vehicles
            .Where(x => x.Id == id)
            .ExecuteDeleteAsync(ct);

        return rowsDeleted == 1
            ? TypedResults.Ok()
            : TypedResults.NotFound();
    }
}
